/* PSPP - computes sample statistics.
   Copyright (C) 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The author may be contacted at <pfaffben@pilot.msu.edu> on the
   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
   through more mundane means. */

#include <config.h>
#include <ctype.h>
#include <assert.h>
#include <limits.h>
#include "common.h"
#include "str.h"

a_string *
aa_strcpy (a_string * d, const a_string * s)
{
  d->l = s->l;
  memcpy (d->s, s->s, s->l);
  return d;
}

a_string *
ab_strcpy (a_string * d, const b_string * s)
{
  d->l = s->l;
  memcpy (d->s, s->s, s->l);
  return d;
}

a_string *
ac_strcpy (a_string * d, const c_string * s)
{
  d->l = LOAD_2 (&s->l1);
  memcpy (d->s, s->s, d->l);
  return d;
}

b_string *
ba_strcpy (b_string * d, const a_string * s)
{
  d->l = s->l;
  memcpy (d->s, s->s, s->l);
  return d;
}

b_string *
bb_strcpy (b_string * d, const b_string * s)
{
  d->l = s->l;
  memcpy (d->s, s->s, s->l);
  return d;
}

c_string *
ca_strcpy (c_string * d, const a_string * s)
{
  assert (s->l <= 65535);
  STORE_2 (&d->l1, s->l);
  memcpy (d->s, s->s, s->l);
  return d;
}

a_string *
aa_strdup (const a_string * s)
{
  a_string *d = xmalloc (sizeof (a_string));
  d->l = s->l;
  d->s = xmalloc (s->l);
  memcpy (d->s, s->s, s->l);
  return d;
}

a_string *
aa_strdupcpy (a_string * d, const a_string * s)
{
  d->l = s->l;
  d->s = xmalloc (s->l);
  memcpy (d->s, s->s, s->l);
  return d;
}

b_string *
ba_strdup (const a_string * s)
{
  b_string *d = xmalloc (sizeof (b_string) + s->l - 1);
  d->l = s->l;
  memcpy (d->s, s->s, s->l);
  return d;
}

char *
sa_strdup (const a_string * s)
{
  char *d = xmalloc (s->l + 1);
  memcpy (d, s->s, s->l);
  d[s->l] = 0;
  return d;
}
